﻿using System;
using System.Threading;

using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

using GHIElectronics.NETMF.FEZ;

using BigBlueSparx.NetMF.FEZ.HMC6352Compass;
using BigBlueSparx.NetMF.FEZ;
using BatronLCD; 

namespace PANDA_II_I2C_HMC6352
{
    public class Program
    {
        public static void Main()
        {
            // Paramètres du bus I2C
            byte addCompass_I2C = 0x22; // Adresse (7 bits) du circuit HMC6352
            byte addLcd_I2C = 0x3B; // Adresse (7 bits) du Lcd I2C Batron
            UInt16 Freq = 100;

            // Création d'un objet boussole HMC6352
            HMC6352Compass compass = new HMC6352Compass(addCompass_I2C,Freq);

            // Création d'un objet Lcd I2C Batron
            I2CLcd BATRON = new I2CLcd(addLcd_I2C, Freq);

            // Initialisation du Lcd I2C
            BATRON.Init();

            // Affichage de la version du software et de l'adresse de la boussole
            float lastHeading = (int)compass.GetHeading();
            byte ver = compass.ReadEeprom(HMC6352Compass.EEPROMAddress.SoftwareVersion);
            Debug.Print("Software Version = " + ver.ToString());
            BATRON.PutString(0,0, "SofVers = " + ver.ToString());
            byte i2cAddr = compass.ReadEeprom(HMC6352Compass.EEPROMAddress.SlaveAddress);
            Debug.Print("Slave Address = " + i2cAddr.ToString());
            BATRON.PutString(0, 1, "2*sla + /W = " + i2cAddr.ToString());

            HMC6352Compass.OperationalMode mode = compass.GetOperationalMode();
            Debug.Print("Operational Mode = " + mode.ToString());

            HMC6352Compass.Frequency frequency = compass.GetFrequency();
            Debug.Print("Frequency = " + frequency.ToString());
            Thread.Sleep(2000);

            while (true)
            {
                Thread.Sleep(500);

                float heading = compass.GetHeading();
                if (heading != lastHeading)
                {
                    BATRON.ClearScreen();
                    BATRON.PutString(0, 0, "Dir: " + heading.ToString());
                }
                lastHeading = heading;
                Debug.Print(heading.ToString());
            } 
        }
    }
}
